//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

// C Source File
// Created 02/02/02; 19:34:39


#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#include <tigcclib.h>         // Include All Header Files

#include "extgraph.h"
#include "tihex.h"

//--------------------------------------------------------------------------

unsigned short Cursor_HEXA[] = {0xFFF0,0xFFF0,0xFFF0,0xFFF0,0xFFF0,0xFFF0,0xFFF0,0xFFF0};

unsigned char Cursor_CHAR[] = {0xFC,0xFC,0xFC,0xFC,0xFC,0xFC,0xFC,0xFC};

//--------------------------------------------------------------------------

unsigned char hex_digits[] = "0123456789ABCDEF";

unsigned char*Adress_LU;
unsigned char*AdressPtr;
int EditX;
int EditY;

int Edit_Mode;

char *Plane0;


//--------------------------------------------------------------------------

// Custom String Input Function
void InputStr (char *buffer, short maxlen)
{
	SCR_STATE ss;
	short key, i = 0;
	buffer[0] = 0;
	SaveScrState (&ss);
	do
	{
		MoveTo (ss.CurX, ss.CurY);
		printf ("%s_  ", buffer);
		// Note that two spaces are required if F_4x6 font is used
		key = ngetchx ();
		if (key >= ' ' && key <= '~' && i < maxlen) buffer[i++] = key;
		if (key == KEY_BACKSPACE && i) i--;
		buffer[i] = 0;
	} while (key != KEY_ENTER);
	
}


//--------------------------------------------------------------------------

void RefreshScreen(void)
{
	int x,y;
	unsigned char * ptr;
	
	unsigned char HexBuff[32];
	HexBuff[16]=0;
	unsigned char StrBuff[32];
	StrBuff[8]=0;
	
	PortSet(Plane0,239,127);
	ClrScr();
	
	ptr = Adress_LU;
	
	for (y=0;y<11;y++)
	{
		
		// HexBuff
		
		for (x=0;x<8;x++)
		{
			HexBuff[(x<<1)+0] = hex_digits[(*ptr)>>4];
			HexBuff[(x<<1)+1] = hex_digits[(*ptr)&15];
			ptr++;
		}
		DrawStr(0,y<<3,HexBuff,A_REPLACE);
		
		// StrBuff
		
		memcpy(StrBuff,ptr-8,8);
		
		for (x=0;x<8;x++)
		if (!isgraph(StrBuff[x]))
		StrBuff[x]='.';
		
		DrawStr(CHAR_LEFT,y<<3,StrBuff,A_REPLACE);
		
	}
	
	// 2700=30*90
	ptr=Plane0+2700;
	int a;
	for (a=0;a<20;a++)
	*ptr++=0xFF;
	
	PortRestore();
	FastCopyScreen(Plane0,LCD_MEM);
	
}
//--------------------------------------------------------------------------


size_t HexStrToLong(unsigned char * str)
{
	int a;
	unsigned char str8[32];
	unsigned char LongData[4];
	unsigned char digit;
	
	sprintf(str8,"%08s",str);
	
	
	for (a=0;a<8;a++)
	{
		
		if (str8[a] >= '0' && str8[a] <= '9')
		digit=str8[a]-'0';
		else if (str8[a] >= 'a' && str8[a] <= 'f')
		digit=str8[a]-'a'+10;
		else if (str8[a] >= 'A' && str8[a] <= 'F')
		digit=str8[a]-'A'+10;
		else
		digit=0;
		
		if (a&1)
		LongData[a>>1]|=digit;
		else
		LongData[a>>1]=(digit<<4);
		
	}
	
	return *(size_t*)LongData;
}
//--------------------------------------------------------------------------


void WriteKey(short key)
{
	unsigned char Value = 0;
	unsigned char digit;
	
	
	if (Edit_Mode==EM_CHAR )
	{
		if (key >= ' ' && key <= '~')
		{
			Value = key;
			WriteValue(Value);
		}
	}
	
	else if (Edit_Mode==EM_HEXA )
	{
		
		if (key >= '0' && key <= '9')
		digit=key-'0';
		else if (key >= 'a' && key <= 'f')
		digit=key-'a'+10;
		else if (key >= 'A' && key <= 'F')
		digit=key-'A'+10;
		else
		return;
		
		Value = (*AdressPtr)<<4;
		Value|=digit;
		WriteValue(Value);
		
	}
	
	
}
//--------------------------------------------------------------------------

void WriteValue(unsigned char Value)
{
	
	TRY
	{
		*AdressPtr = Value;
	}
	ONERR
	{
		
		clrscr();
		printf("
		Error on writing
		Code : %hd",errCode);
		ngetchx();
		
		RefreshScreen();
		
	}
	ENDTRY
	
}
//--------------------------------------------------------------------------

void OpenFileMenu(void)
{
	char buffer[128];
	short key;
	int LabelTop=0;
	
	SYM_ENTRY *SymPtr_Folder;
	SYM_ENTRY *SymPtr_File;
	char symname_folder[16];
	char symname_file[16];
	
	
	
	symname_folder[8]=0;
	symname_file[8]=0;
	
	SymPtr_Folder=NULL;
	
	ClrScr();
	DrawStr(0,LabelTop,"Open File",A_REPLACE);
	LabelTop+=8;
	
	
	for (;;)
	{
		
		if (!SymPtr_Folder)
		SymPtr_Folder=SymFindFirst (NULL, 0);
		
		memcpy(symname_folder,SymPtr_Folder->name,8);
		sprintf(buffer,"Folder   : <%8s> ",symname_folder);
		
		DrawStr(0,LabelTop,buffer,A_REPLACE);
		
		key = ngetchx();
		
		if (key==KEY_LEFT)
		SymPtr_Folder=SymFindPrev ();
		else if (key==KEY_RIGHT)
		SymPtr_Folder=SymFindNext ();
		else if (key==KEY_ENTER)
		break;
		
	}
	
	LabelTop+=8;
	
	SymPtr_File=SymFindFirst (SYMSTR(symname_folder), 1);
	
	if (!SymPtr_File)
	{
		DrawStr(0,LabelTop,"No Files",A_REPLACE);
		LabelTop+=8;
		
		ngetchx();
		return;
	}
	
	for (;;)
	{
		
		if (!SymPtr_File)
		SymPtr_File=SymFindFirst (SYMSTR(symname_folder), 1);
		
		memcpy(symname_file,SymPtr_File->name,8);
		sprintf(buffer,"FileName : <%8s> ",symname_file);
		
		DrawStr(0,LabelTop,buffer,A_REPLACE);
		
		key = ngetchx();
		
		if (key==KEY_LEFT)
		SymPtr_File=SymFindPrev ();
		else if (key==KEY_RIGHT)
		SymPtr_File=SymFindNext ();
		else if (key==KEY_ENTER)
		break;
		
	}
	
	GotoAddr((char*)HeapDeref (SymPtr_File->handle));
	
}
//--------------------------------------------------------------------------

void GotoAddr (unsigned char*ptr)
{
	
	Adress_LU = ptr;
	AdressPtr=Adress_LU;
	EditX=0;
	EditY=0;
	
}
//--------------------------------------------------------------------------

// Main Function
void _main(void)
{
	char buffer[128];
	short key;
	
	if (!(Plane0=malloc(LCD_SIZE)))
	goto end_prog;
	
	
	
	FontSetSys(0);
	clrscr();
	printf("TIHex 1.3 \n");
	printf("Programmed by Olivier Sangala \n");
	printf("http://olivier.sangala.free.fr \n");
	printf("olivier_sangala@hotmail.com \n");
	FontSetSys(1);
	ngetchx();
	
	
	Edit_Mode = EM_CHAR;
	
	GotoAddr(0);
	RefreshScreen();
	
	for (;;)
	{
		
		
		AdressPtr=Adress_LU+(EditY<<3)+EditX;
		
		sprintf(buffer,"%08lX",(size_t)AdressPtr);
		DrawStr(0,STATUS_ADDR_TOP,buffer,A_REPLACE);
		
		if (Edit_Mode == EM_HEXA)
		DrawStr(CHAR_LEFT,STATUS_ADDR_TOP,"HEXA",A_REPLACE);
		else
		DrawStr(CHAR_LEFT,STATUS_ADDR_TOP,"CHAR",A_REPLACE);
		
		
		Sprite16_XOR((EditX<<3)+(EditX<<2), EditY<<3, 8, Cursor_HEXA, LCD_MEM);
		Sprite8_XOR(CHAR_LEFT+(EditX<<2)+(EditX<<1), EditY<<3, 8, Cursor_CHAR, LCD_MEM);
		
		key = ngetchx();
		
		Sprite16_XOR((EditX<<3)+(EditX<<2), EditY<<3, 8, Cursor_HEXA, LCD_MEM);
		Sprite8_XOR(CHAR_LEFT+(EditX<<2)+(EditX<<1), EditY<<3, 8, Cursor_CHAR, LCD_MEM);
		
		
		if (key==KEY_DOWN)
		{
			EditY++;
			if (EditY>10)
			{
				EditY=10;
				Adress_LU+=8;
				RefreshScreen();
			}
		}
		
		else if (key==KEY_UP)
		{
			EditY--;
			if (EditY<0)
			{
				EditY=0;
				Adress_LU-=8;
				RefreshScreen();
			}
		}
		
		else if (key==KEY_RIGHT)
		{
			EditX++;
			EditX&=7;
		}
		
		else if (key==KEY_LEFT)
		{
			EditX--;
			EditX&=7;
		}
		
		else if (key==((signed)KEY_SHIFT+KEY_DOWN))
		{
			Adress_LU+=8*11;
			AdressPtr+=8*11;
			RefreshScreen();
		}
		
		else if (key==((signed)KEY_SHIFT+KEY_UP))
		{
			Adress_LU-=8*11;
			AdressPtr-=8*11;
			RefreshScreen();
		}
		
		else if (key==(KEY_F1))
		{
			
			clrscr();
			printf("Goto Adress :\n");
			InputStr(buffer,8);
			
			GotoAddr((char*)HexStrToLong(buffer));
			RefreshScreen();
			
		}
		
		else if (key==(KEY_F2))
		{
			OpenFileMenu();
			RefreshScreen();
		}
		
		
		else if (key==(KEY_CLEAR))
		{
			WriteValue(0);
			RefreshScreen();
		}
		
		else if (key==(KEY_APPS))
		{
			off();
		}
		
		else if (key==(KEY_MODE))
		{
			if (Edit_Mode == EM_HEXA)
			Edit_Mode=EM_CHAR;
			else
			Edit_Mode=EM_HEXA;
		}
		
		
		else if (key==(KEY_ESC))
		{
			
			clrscr();
			
			printf("[ESC]   : Continue \n");
			printf("[ENTER] : Exit \n");
			
			while ((key = ngetchx())!=KEY_ENTER && key!=KEY_ESC);
			
			if (key==KEY_ENTER)
			break;
			
			RefreshScreen();
			
		}
		
		else
		{
			WriteKey(key);
			RefreshScreen();
		}
		
	}
	
	end_prog:
	
	if (Plane0)
	free(Plane0);
	
}
